"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var index_exports = {};
__export(index_exports, {
  ScrollSync: () => ScrollSync,
  ScrollSyncPane: () => ScrollSyncPane
});
module.exports = __toCommonJS(index_exports);

// src/components/ScrollSync.tsx
var import_react2 = __toESM(require("react"), 1);

// src/hooks/useScrollSyncContext.ts
var import_react = require("react");
var ScrollSyncContext = (0, import_react.createContext)(void 0);
var useScrollSyncContext = () => {
  const context = (0, import_react.useContext)(ScrollSyncContext);
  if (!context) {
    throw new Error(
      "useScrollSyncContext must be used within a ScrollSyncProvider"
    );
  }
  return context;
};

// src/components/ScrollSync.tsx
var import_jsx_runtime = require("react/jsx-runtime");
var ScrollSync = ({
  children,
  enabled = true,
  horizontal = true,
  onSync,
  proportional = true,
  vertical = true
}) => {
  const panesRef = (0, import_react2.useRef)({});
  const findPane = (0, import_react2.useCallback)((node, group) => {
    if (!panesRef.current[group]) {
      return false;
    }
    return panesRef.current[group].find((pane) => pane === node);
  }, []);
  const syncScrollPosition = (0, import_react2.useCallback)(
    (scrolledPane, pane) => {
      const {
        clientHeight,
        clientWidth,
        scrollHeight,
        scrollLeft,
        scrollTop,
        scrollWidth
      } = scrolledPane;
      const scrollTopOffset = scrollHeight - clientHeight;
      const scrollLeftOffset = scrollWidth - clientWidth;
      const paneHeight = pane.scrollHeight - clientHeight;
      const paneWidth = pane.scrollWidth - clientWidth;
      if (vertical && scrollTopOffset > 0) {
        pane.scrollTop = proportional ? paneHeight * scrollTop / scrollTopOffset : scrollTop;
      }
      if (horizontal && scrollLeftOffset > 0) {
        pane.scrollLeft = proportional ? paneWidth * scrollLeft / scrollLeftOffset : scrollLeft;
      }
    },
    [proportional, vertical, horizontal]
  );
  const removeEvents = (0, import_react2.useCallback)((node) => {
    node.onscroll = null;
  }, []);
  const addEvents = (0, import_react2.useCallback)(
    (node, groups) => {
      node.onscroll = () => {
        if (!enabled) return;
        window.requestAnimationFrame(() => {
          groups.forEach((group) => {
            panesRef.current[group]?.forEach((pane) => {
              if (node !== pane) {
                removeEvents(pane);
                syncScrollPosition(node, pane);
                window.requestAnimationFrame(() => {
                  const paneGroups = Object.keys(panesRef.current).filter(
                    (paneGroup) => panesRef.current[paneGroup].includes(pane)
                  );
                  addEvents(pane, paneGroups);
                });
              }
            });
          });
        });
        if (onSync) {
          onSync(node);
        }
      };
    },
    [onSync, removeEvents, syncScrollPosition, enabled]
  );
  const registerPane = (0, import_react2.useCallback)(
    (node, groups) => {
      groups.forEach((group) => {
        if (!panesRef.current[group]) {
          panesRef.current[group] = [];
        }
        if (!findPane(node, group)) {
          if (panesRef.current[group].length > 0) {
            syncScrollPosition(panesRef.current[group][0], node);
          }
          panesRef.current[group].push(node);
        }
      });
      addEvents(node, groups);
    },
    [findPane, syncScrollPosition, addEvents]
  );
  const unregisterPane = (0, import_react2.useCallback)(
    (node, groups) => {
      groups.forEach((group) => {
        if (findPane(node, group)) {
          removeEvents(node);
          const index = panesRef.current[group].indexOf(node);
          if (index !== -1) {
            panesRef.current[group].splice(index, 1);
          }
        }
      });
    },
    [findPane, removeEvents]
  );
  return /* @__PURE__ */ (0, import_jsx_runtime.jsx)(ScrollSyncContext.Provider, { value: { registerPane, unregisterPane }, children: import_react2.default.Children.only(children) });
};

// src/components/ScrollSyncPane.tsx
var import_react3 = require("react");
var castArray = (groups) => Array.isArray(groups) ? groups : [groups];
var ScrollSyncPane = ({
  attachTo,
  children,
  enabled = true,
  group = "default",
  innerRef
}) => {
  const context = useScrollSyncContext();
  const childRef = (0, import_react3.useRef)(null);
  const nodeRef = (0, import_react3.useRef)(null);
  const updateNode = (0, import_react3.useCallback)(() => {
    if (attachTo) {
      nodeRef.current = typeof attachTo === "function" ? null : attachTo.current;
    } else {
      nodeRef.current = childRef.current;
    }
  }, [attachTo]);
  (0, import_react3.useEffect)(() => {
    updateNode();
    if (enabled && nodeRef.current) {
      context.registerPane(nodeRef.current, castArray(group));
    }
    return () => {
      if (enabled && nodeRef.current) {
        context.unregisterPane(nodeRef.current, castArray(group));
      }
    };
  }, [context, enabled, group, attachTo, updateNode]);
  if (attachTo) {
    return children;
  }
  return (0, import_react3.cloneElement)(children, {
    ref: (node) => {
      childRef.current = node;
      if (typeof innerRef === "function") {
        innerRef(node);
      } else if (innerRef && node) {
        innerRef.current = node;
      }
    }
  });
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  ScrollSync,
  ScrollSyncPane
});
//# sourceMappingURL=index.cjs.map